--=============================================================================
-- Project: Video quality analysis using an FPGA
-- Author: Eirik Trud Nordeng
-------------------------------------------------------------------------------
-- Description: The file contains VHDL code describing the functionality of the
-- video quality assessment system for calculating the A & D metric for a
-- single color channel.
--=============================================================================
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use ieee.numeric_std.all;

entity ad is
	generic(
		channel_width : integer := 8;
		rows          : integer := 1200;
		cols          : integer := 1920
		);
	port (
		clk          : in  std_logic;
		reset       : in  std_logic;
		
		vsync : in std_logic;
		hsync : in std_logic;
		
		data_in   : in  integer range 0 to (2**channel_width - 1);
		
		data_out   : out integer range 0 to 2**16 - 1
		);
end ad;

architecture behaviour of ad is	 
	
	constant average_number_of_frames : integer := 8;
	
	-- Component declaration of the "shift_register(behaviour)" unit defined in
	-- file: "./../src/shift_register.vhd"
	component shift_register
		generic(
			data_width : INTEGER := 16;
			size : INTEGER := 1200);
		port(
			clk : in STD_LOGIC;
			reset : in STD_LOGIC;
			data_enable : in std_logic;
			data_in : in INTEGER range 0 to 2**data_width-1;
			data_out : out INTEGER range 0 to 2**data_width-1);
	end component;
	for all: shift_register use entity work.shift_register(behaviour);
	
	
	-- Component declaration of the "averaging(behaviour)" unit defined in
	-- file: "./../src/averaging.vhd"
	component averaging
		generic(
			metric_max_value : INTEGER := 65535;
			number_of_samples : INTEGER := 8);
		port(
			clk : in STD_LOGIC;
			reset : in STD_LOGIC;
			enable : in STD_LOGIC;
			data_in : in INTEGER range 0 to metric_max_value;
			data_out : out INTEGER range 0 to metric_max_value);
	end component;
	for all: averaging use entity work.averaging(behaviour);	
	
	signal accumulate_reg : integer range 0 to (2**16 - 1);
	signal from_shift_reg : integer range 0 to (2**16 - 1);
	signal from_sad : integer range 0 to (2**16 - 1);
	signal vsync_flank_trigger : std_logic;	
	signal hsync_flank_trigger : std_logic;
	signal enable_averaging : std_logic;
	signal enable_shift_reg : std_logic;
begin
	
	shift_reg_component : shift_register
	generic map(
		data_width => 16,
		size => 1200
		)
	port map(
		clk => clk,
		reset => reset,
		data_enable => enable_shift_reg,
		data_in => accumulate_reg,
		data_out => from_shift_reg
		);
	
	accumulate_proc : process(clk)
	begin  
		if (rising_edge(clk)) then			
			if (vsync = '1') then -- New frame
				-- Flank trigger 
				if (vsync_flank_trigger = '1') then
					from_sad <= 0; 
					enable_averaging <= '0';
				else   
					from_sad <= abs(accumulate_reg - from_shift_reg);
					enable_averaging <= '1';
					vsync_flank_trigger <= '1';
				end if;
			else
				from_sad <= 0;
				enable_averaging <= '0';
				vsync_flank_trigger <= '0';
			end if;
			
			if (hsync = '1' and vsync = '0') then -- New line
				-- Flank trigger
				if (hsync_flank_trigger = '1') then	
					enable_shift_reg <= '0';
					accumulate_reg <= 0;
					
				else
					enable_shift_reg <= '1';
					accumulate_reg <= 0;
					hsync_flank_trigger <= '1';	
				end if;
			elsif(hsync = '0' and vsync = '0') then
				accumulate_reg <= accumulate_reg + data_in;
				hsync_flank_trigger <= '0';
			end if;
		end if;
	end process accumulate_proc;
	
		row_averaging_comp : averaging
		generic map(
			metric_max_value => 2**16 - 1,
			number_of_samples => 32
		)
		port map(
			clk => clk,
			reset => reset,
			enable => enable_averaging,
			data_in => from_sad,
			data_out => data_out
		);
	
end behaviour;
