library IEEE;					
use IEEE.STD_LOGIC_1164.all;

entity histogram_accumulator is
	generic (						 
		channel_width : integer := 8;
		--rows          : integer := 1080;
		--cols          : integer := 1920;
		bin_max_value : integer := 4095;
		number_of_bins : integer := 256
		);
	port (		
		clk : in std_logic;
		reset : in std_logic; 
		
		index : in integer range 0 to number_of_bins - 1;
		data_in : in integer range 0 to 2**channel_width - 1;
		
		count : out INTEGER range 0 to bin_max_value
		);
end histogram_accumulator;				   

architecture behaviour of histogram_accumulator is
	
	constant size_of_bins : integer := (2**channel_width)/number_of_bins;
	constant channel_max_value : integer := 2**channel_width - 1;
	
	-- All counter registers may be reset in parallell.
	-- Parallell access of the data is possible.
	-- Can use the pixel clock.
	
	-- Component declaration of the "counter(behaviour)" unit defined in
	-- file: "./../src/counter.vhd"
	component counter
		generic(
			channel_width : INTEGER := 8;
			max_value : integer := 4095
			);
		port(
			clk : in STD_LOGIC;
			reset : in STD_LOGIC;
			clock_enable : in STD_LOGIC;
			data_out : out INTEGER range 0 to bin_max_value);
	end component;
	for all: counter use entity work.counter(behaviour);
	
	type clock_enable_array_t is array(0 to number_of_bins - 1) of std_logic;
	signal clock_enable_array : clock_enable_array_t;		
	
	type data_out_array_t is array(0 to number_of_bins - 1) of integer range 0 to bin_max_value;
	signal data_out_array : data_out_array_t;
	
	subtype boundary_integer_t is integer range 0 to channel_max_value; 
	
	function calculate_upper_boundary(index : in integer range 0 to (number_of_bins - 1)) return boundary_integer_t is
	begin
		return ((size_of_bins * (index + 1)) - 1);
	end calculate_upper_boundary; 
	
	function calculate_lower_boundary(index : in integer range 0 to (number_of_bins - 1)) return boundary_integer_t is
	begin
		return size_of_bins * index;
	end calculate_lower_boundary;
	
begin
	
	decoder_in_proc : process(data_in)
	begin
		for i in 0 to number_of_bins - 1 loop
			if (data_in >= calculate_lower_boundary(i)) and (data_in <= calculate_upper_boundary(i)) then
				clock_enable_array(i) <= '1';
			else
				clock_enable_array(i) <= '0';
			end if;
		end loop;
	end process decoder_in_proc;
	
	counters : for i in 0 to number_of_bins - 1 generate
		counter_component : counter
		generic map(
			channel_width => channel_width,
			max_value => bin_max_value
			)
		port map(
			clk => clk,
			reset => reset,
			clock_enable => clock_enable_array(i),
			data_out => data_out_array(i)
			);
	end generate;
	
	multiplexer_out_proc : process(index, data_out_array)
	begin
		count <= data_out_array(index);
	end process multiplexer_out_proc;
	
end behaviour;