--=============================================================================
-- Project: Video quality analysis using an FPGA
-- Copyright: Eirik Trud Nordeng, Cisco
-- Author: Eirik Trud Nordeng
-- Revision: 1 
-- Last revised: 2013 - 01 - 07
-- Workfile: 
-- Archive: 
-------------------------------------------------------------------------------
-- Description:	This file contains the code for a video metric system that
-- is intended to detect blurred images. It is written as a part of the 
-- master project of Eirik Trud Nordeng at IET, NTNU.
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- Revision History:
-- 2013 - 01 - 07: File created and entity created.
-- 2013 - 02 - 28: Gives warning when signals are unused. This is OK.
-- 
--=============================================================================
library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity convolution_filter_3x3 is	
	generic(	
		-- Channel bit width:
		rows : integer := 512;
		cols : integer := 512;
		channel_width : integer := 8	
		);
	port(		
		clk : in std_logic;
		reset : in std_logic;
		
		hsync : in std_logic;
		vsync : in std_logic;
		
		data_in : in integer range 0 to (2**channel_width - 1);
		 
		data_out : out integer range -1020 to 1020
		);
end entity convolution_filter_3x3;

architecture begaviour of convolution_filter_3x3 is  

	-- Component declaration of the "row_buffers(behaviour)" unit defined in
	-- file: "./../src/row_buffers.vhd"
	component row_buffers
	generic(
		rows : INTEGER := 512;
		cols : INTEGER := 512;
		channel_width : INTEGER := 8);
	port(
		clk : in STD_LOGIC;
		reset : in STD_LOGIC;
		hsync : in STD_LOGIC;
		vsync : in STD_LOGIC;
		data_in : in INTEGER range 0 to 2**channel_width-1;
		data_out_1 : out INTEGER range 0 to 2**channel_width-1;
		data_out_2 : out INTEGER range 0 to 2**channel_width-1;
		data_out_3 : out INTEGER range 0 to 2**channel_width-1);
	end component;
	for all: row_buffers use entity work.row_buffers(behaviour);
	
	signal data_from_buffer_1 : integer range 0 to (2**channel_width - 1); 
	signal data_from_buffer_2 : integer range 0 to (2**channel_width - 1);
	signal data_from_buffer_3 : integer range 0 to (2**channel_width - 1);
		
	-- Component declaration of the "filter(behaviour)" unit defined in
	-- file: "./../src/filter.vhd"
	component filter
	generic(
		channel_width : INTEGER := 8;
		t_1_1 : INTEGER := 0;
		t_1_2 : INTEGER := 1;
		t_1_3 : INTEGER := 0;
		t_2_1 : INTEGER := 1;
		t_2_2 : INTEGER := -4;
		t_2_3 : INTEGER := 1;
		t_3_1 : INTEGER := 0;
		t_3_2 : INTEGER := 1;
		t_3_3 : INTEGER := 0);
	port(
		clk : in STD_LOGIC;
		reset : in STD_LOGIC;
		vsync : in STD_LOGIC;
		hsync : in STD_LOGIC;
		data_1 : in INTEGER range 0 to 2**channel_width-1;
		data_2 : in INTEGER range 0 to 2**channel_width-1;
		data_3 : in INTEGER range 0 to 2**channel_width-1;
		data_out : out INTEGER range -1020 to 1020);
	end component;
	for all: filter use entity work.filter(behaviour);	
		
begin	 

	row_buffers_component : row_buffers
	generic map(
		rows => rows,
		cols => cols,
		channel_width => channel_width
	)
	port map(
		clk => clk,
		reset => reset,
		hsync => hsync,
		vsync => vsync,
		data_in => data_in,
		data_out_1 => data_from_buffer_1,
		data_out_2 => data_from_buffer_2,
		data_out_3 => data_from_buffer_3
	);

	filter_component : filter
	generic map(
		channel_width => channel_width
	)
	port map(
		clk => clk,
		reset => reset,
		vsync => vsync,
		hsync => hsync,
		data_1 => data_from_buffer_1,
		data_2 => data_from_buffer_2,
		data_3 => data_from_buffer_3,
		data_out => data_out
	);	
	
end architecture begaviour;