--=============================================================================
-- Project: Video quality analysis using an FPGA
-- Author: Eirik T�rud Nordeng
--=============================================================================
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use ieee.numeric_std.all;

entity vqa_blurMetric_laplace is
	generic(
		channel_width : integer := 8;
		rows          : integer := 1200;
		cols          : integer := 1920
		);
	port (
		clk          : in  std_logic;
		reset       : in  std_logic;
		
		vsync : in std_logic;
		hsync : in std_logic;
		
		red   : in  integer range 0 to (2**channel_width - 1);
		green : in  integer range 0 to (2**channel_width - 1);
		blue  : in  integer range 0 to (2**channel_width - 1);
		
		metric_red   : out integer range 0 to 1020;
		metric_green : out integer range 0 to 1020;
		metric_blue  : out integer range 0 to 1020
		);
end vqa_blurMetric_laplace;

architecture behaviour of vqa_blurMetric_laplace is	 
	
	constant metric_max_value : integer := 1020;
	constant number_of_samples : integer := 8;
	
	-- Component declaration of the "counter(behaviour)" unit defined in
	-- file: "./../src/counter.vhd"
	component counter
		generic(
			channel_width : INTEGER := 8;
			max_value : INTEGER := 4095);
		port(
			clk : in STD_LOGIC;
			reset : in STD_LOGIC;
			clock_enable : in STD_LOGIC;
			data_out : out INTEGER range 0 to max_value);
	end component;
	for all: counter use entity work.counter(behaviour);
	
	
	-- Component declaration of the "convolution_filter_3x3(begaviour)" unit defined in
	-- file: "./../src/convolution_filter_3x3.vhd"
	component convolution_filter_3x3
		generic(
			rows : INTEGER := 1080;
			cols : INTEGER := 1920;
			channel_width : INTEGER := 8);
		port(
			clk : in STD_LOGIC;
			reset : in STD_LOGIC;
			hsync : in STD_LOGIC;
			vsync : in STD_LOGIC;
			data_in : in INTEGER range 0 to 2**channel_width-1;
			data_out : out INTEGER range -1020 to 1020);
	end component;
	for all: convolution_filter_3x3 use entity work.convolution_filter_3x3(begaviour);
	
	-- Component declaration of the "comparator(behaviour)" unit defined in
	-- file: "./../src/comparator.vhd"
	component comparator
		generic(
			channel_width : INTEGER := 8);
		port(
			clk : in STD_LOGIC;
			reset : in STD_LOGIC;
			data_in : in INTEGER range -1020 to 1020;
			data_out : out INTEGER range 0 to 1020);
	end component;
	for all: comparator use entity work.comparator(behaviour);
	
	-- Component declaration of the "averaging(behaviour)" unit defined in
	-- file: "./../src/averaging.vhd"
	component averaging
		generic(
			rows : INTEGER := 1080;
			cols : INTEGER := 1920;
			metric_max_value : INTEGER := 65535;
			number_of_samples : INTEGER := 32);
		port(
			clk : in STD_LOGIC;
			reset : in STD_LOGIC;
			enable : in STD_LOGIC;
			data_in : in INTEGER range 0 to metric_max_value;
			data_out : out INTEGER range 0 to metric_max_value);
	end component;
	for all: averaging use entity work.averaging(behaviour);	
	
	signal row_count_enable : std_logic;
	signal col_count_enable : std_logic;
	
	signal current_row : integer range 1 to rows;
	signal current_col : integer range 1 to cols;
	
	signal raw_metric_red   : integer range -1020 to 1020;
	signal raw_metric_green : integer range -1020 to 1020;
	signal raw_metric_blue  : integer range -1020 to 1020;
	
	-- Pipelining:
	signal to_red_abs_reg : integer range -1020 to 1020;
	signal to_green_abs_reg : integer range -1020 to 1020;
	signal to_blue_abs_reg : integer range -1020 to 1020;
	
	signal to_red_comparator : integer range 0 to 1020;
	signal to_green_comparator : integer range 0 to 1020;
	signal to_blue_comparator : integer range 0 to 1020;
	
	signal enable_averaging : std_logic;
	
	signal to_red_averaging : integer range 0 to 1020;
	signal to_green_averaging : integer range 0 to 1020;
	signal to_blue_averaging : integer range 0 to 1020;
	
	signal averaging_flank_trigger : std_logic;
	signal vsync_flank_trigger : std_logic;
	signal hsync_flank_trigger : std_logic;
	
begin
	
	enable_counters_proc : process(clk)
	begin	
		if (rising_edge(clk)) then
			
			if (vsync = '1') then
				if (vsync_flank_trigger = '1') then
					row_count_enable <= '0';
				else   
					row_count_enable <= '1';
					vsync_flank_trigger <= '1';
				end if;
			else
				row_count_enable <= '0';
				vsync_flank_trigger <= '0';
			end if;
			
			if (hsync = '1') then
				if (hsync_flank_trigger = '1') then
					col_count_enable <= '0';
				else
					col_count_enable <= '1';
					hsync_flank_trigger <= '1';
				end if;
			else
				col_count_enable <= '0';
				hsync_flank_trigger <= '0';
			end if;
		end if;
		
	end process enable_counters_proc;
	
	row_counter : counter
	generic map(
		channel_width => channel_width,
		max_value => rows - 1
		)
	port map(
		clk => clk,
		reset => reset,
		clock_enable => row_count_enable,
		data_out => current_row
		);
	
	coloumn_counter : counter
	generic map(
		channel_width => channel_width,
		max_value => cols - 1
		)
	port map(
		clk => clk,
		reset => reset,
		clock_enable => col_count_enable,
		data_out => current_col
		);
	
	convolution_red : convolution_filter_3x3
	generic map(
		rows => rows,
		cols => cols,
		channel_width => channel_width
		)
	port map(
		clk => clk,
		reset => reset,
		hsync => hsync,
		vsync => vsync,
		data_in => red,
		data_out => raw_metric_red
		);  
	
	convolution_green : convolution_filter_3x3
	generic map(
		rows => rows,
		cols => cols,
		channel_width => channel_width
		)
	port map(
		clk => clk,
		reset => reset,
		hsync => hsync,
		vsync => vsync,
		data_in => green,
		data_out => raw_metric_green
		); 
	
	convolution_blue : convolution_filter_3x3
	generic map(
		rows => rows,
		cols => cols,
		channel_width => channel_width
		)
	port map(
		clk => clk,
		reset => reset,
		hsync => hsync,
		vsync => vsync,
		data_in => blue,
		data_out => raw_metric_blue
		); 
	
	pipelining_proc : process(clk)
	begin						 
		if (rising_edge(clk)) then
			if (reset = '1') then
				to_red_abs_reg <= 0;
				to_green_abs_reg <= 0;
				to_blue_abs_reg <= 0;
--			elsif (current_row = rows - 1 and current_col = cols - 1) then
--				to_red_abs_reg <= 0;
--				to_green_abs_reg <= 0;
--				to_blue_abs_reg <= 0;				
			else
				to_red_abs_reg <= raw_metric_red;
				to_green_abs_reg <= raw_metric_green;
				to_blue_abs_reg <= raw_metric_blue;
			end if;	  
		end if;
	end process pipelining_proc;
	
	abs_proc_red : process(to_red_abs_reg)
	begin
		to_red_comparator <= abs(to_red_abs_reg);
	end process;
	
	abs_proc_green : process(to_green_abs_reg)
	begin
		to_green_comparator <= abs(to_green_abs_reg);
	end process;
	
	abs_proc_blue : process(to_blue_abs_reg)
	begin
		to_blue_comparator <= abs(to_blue_abs_reg);
	end process;
	
	comparator_red : comparator
	generic map(
		channel_width => channel_width
		)
	port map(
		clk => clk,
		reset => reset,
		data_in => to_red_comparator,
		data_out => to_red_averaging
		); 
	
	comparator_green : comparator
	generic map(
		channel_width => channel_width
		)
	port map(
		clk => clk,
		reset => reset,
		data_in => to_green_comparator,
		data_out => to_green_averaging
		);
	
	comparator_blue : comparator
	generic map(
		channel_width => channel_width
		)
	port map(
		clk => clk,
		reset => reset,
		data_in => to_blue_comparator,
		data_out => to_blue_averaging
		);
	
	enable_averaging_proc : process (clk)
	begin
		if (rising_edge(clk)) then
			if (vsync = '1') then
				if (averaging_flank_trigger = '1') then
					enable_averaging <= '0';
					averaging_flank_trigger <= '1';
				else   
					enable_averaging <= '1';
					averaging_flank_trigger <= '1';
				end if;
			else
				enable_averaging <= '0';
				averaging_flank_trigger <= '0';
			end if;	
		end if;
	end process enable_averaging_proc;		
		
	average_red : averaging
	generic map(
		rows => rows,
		cols => cols,
		metric_max_value => metric_max_value,
		number_of_samples => number_of_samples
		)
	port map(
		clk => clk,
		reset => reset,
		enable => enable_averaging,
		data_in => to_red_averaging,
		data_out => metric_red
		);			 
	
	average_green : averaging
	generic map(
		rows => rows,
		cols => cols,
		metric_max_value => metric_max_value,
		number_of_samples => number_of_samples
		)
	port map(
		clk => clk,
		reset => reset,
		enable => enable_averaging,
		data_in => to_green_averaging,
		data_out => metric_green
		);			
	
	average_blue : averaging
	generic map(
		rows => rows,
		cols => cols,
		metric_max_value => metric_max_value,
		number_of_samples => number_of_samples
		)
	port map(
		clk => clk,
		reset => reset,
		enable => enable_averaging,
		data_in => to_blue_averaging,
		data_out => metric_blue
		);
	
end behaviour;

