--=============================================================================
-- Project: Video quality analysis using an FPGA
-- Author: Eirik T�rud Nordeng
--=============================================================================
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use ieee.numeric_std.all;

entity median_filter is
	generic(
		channel_width : integer := 8;
		rows          : integer := 1080;
		cols          : integer := 1920
		);
	port (
		clk          : in  std_logic;
		reset       : in  std_logic;
		
		vsync : in std_logic;
		hsync : in std_logic;
		
		data_in   : in  integer range 0 to (2**channel_width - 1);
		
		original_data_out : out integer;
		filtered_data_out  : out integer
		);
end median_filter;

architecture behaviour of median_filter is

	constant max_channel_value : integer := 2**channel_width-1;

	-- Component declaration of the "row_buffers(behaviour)" unit defined in
	-- file: "./../src/row_buffers.vhd"
	component row_buffers
	generic(
		rows : INTEGER := 1080;
		cols : INTEGER := 1920;
		channel_width : INTEGER := 8);
	port(
		clk : in STD_LOGIC;
		reset : in STD_LOGIC;
		hsync : in STD_LOGIC;
		vsync : in STD_LOGIC;
		data_in : in INTEGER range 0 to 2**channel_width-1;
		data_out_1 : out INTEGER range 0 to 2**channel_width-1;
		data_out_2 : out INTEGER range 0 to 2**channel_width-1;
		data_out_3 : out INTEGER range 0 to 2**channel_width-1);
	end component;
	for all: row_buffers use entity work.row_buffers(behaviour); 
		
	-- Component declaration of the "sorter_3(behaviour)" unit defined in
	-- file: "./../src/sorter_3.vhd"
	component sorter_3
	generic(
		channel_width : INTEGER := 8);
	port(
		clk : in STD_LOGIC;
		reset : in STD_LOGIC;
		upper_data_in : in INTEGER range 0 to 2**channel_width-1;
		middle_data_in : in INTEGER range 0 to 2**channel_width-1;
		lower_data_in : in INTEGER range 0 to 2**channel_width-1;
		upper_data_out : out INTEGER range 0 to 2**channel_width-1;
		middle_data_out : out INTEGER range 0 to 2**channel_width-1;
		lower_data_out : out INTEGER range 0 to 2**channel_width-1);
	end component;
	for all: sorter_3 use entity work.sorter_3(behaviour);
   
	signal to_window_1 : integer range 0 to max_channel_value; 	
	signal to_window_2 : integer range 0 to max_channel_value;
	signal to_window_3 : integer range 0 to max_channel_value;
	
	signal from_first_sorter_1 : integer range 0 to max_channel_value; 	
	signal from_first_sorter_2 : integer range 0 to max_channel_value;
	signal from_first_sorter_3 : integer range 0 to max_channel_value;
	
	signal from_first_pipeline_1 : integer range 0 to max_channel_value; 	
	signal from_first_pipeline_2 : integer range 0 to max_channel_value;
	signal from_first_pipeline_3 : integer range 0 to max_channel_value;
	
	signal from_second_pipeline_1 : integer range 0 to max_channel_value; 	
	signal from_second_pipeline_2 : integer range 0 to max_channel_value;
	signal from_second_pipeline_3 : integer range 0 to max_channel_value;
	
	signal from_second_sorter_1 : integer range 0 to max_channel_value; 	
	signal from_second_sorter_2 : integer range 0 to max_channel_value;
	signal from_second_sorter_3 : integer range 0 to max_channel_value;
	
	signal from_third_sorter_1 : integer range 0 to max_channel_value; 	
	signal from_third_sorter_2 : integer range 0 to max_channel_value;
	signal from_third_sorter_3 : integer range 0 to max_channel_value;
	
	signal from_forth_sorter_1 : integer range 0 to max_channel_value; 	
	signal from_forth_sorter_2 : integer range 0 to max_channel_value;
	signal from_forth_sorter_3 : integer range 0 to max_channel_value;
	
	signal dummy_out_1 : integer range 0 to max_channel_value;
	signal dummy_out_2 : integer range 0 to max_channel_value;
		
begin
	
	row_buffer_component : row_buffers
	generic map(
		rows => rows,
		cols => cols,
		channel_width => channel_width
	)
	port map(
		clk => clk,
		reset => reset,
		hsync => hsync,
		vsync => vsync,
		data_in => data_in,
		data_out_1 => to_window_1,
		data_out_2 => to_window_2,
		data_out_3 => to_window_3
	);
	
	-- Second line in diagram
	
	first_sorter_component : sorter_3
	generic map(
		channel_width => channel_width
	)
	port map(
		clk => clk,
		reset => reset,
		upper_data_in => to_window_1,
		middle_data_in => to_window_2,
		lower_data_in => to_window_3,
		upper_data_out => from_first_sorter_1,
		middle_data_out => from_first_sorter_2,
		lower_data_out => from_first_sorter_3
	);
	
	first_pipelining_proc : process(clk)
	begin
		if (rising_edge(clk)) then
			if (reset = '1') then
				from_first_pipeline_1 <= 0;
				from_first_pipeline_2 <= 0;
				from_first_pipeline_3 <= 0;
			else
				from_first_pipeline_1 <= from_first_sorter_1;
				from_first_pipeline_2 <= from_first_sorter_2;
				from_first_pipeline_3 <= from_first_sorter_3;
			end if;
		end if;
	end process first_pipelining_proc;
	
	second_pipelining_proc : process(clk)
	begin
		if (rising_edge(clk)) then
			if (reset = '1') then
				from_second_pipeline_1 <= 0;
				from_second_pipeline_2 <= 0;
				from_second_pipeline_3 <= 0;
			else
				from_second_pipeline_1 <= from_first_pipeline_1;
				from_second_pipeline_2 <= from_first_pipeline_2;
				from_second_pipeline_3 <= from_first_pipeline_3;
			end if;
		end if;
	end process second_pipelining_proc;
	
	-- Third line in diagram 
	
	second_sorter_component : sorter_3
	generic map(
		channel_width => channel_width
	)
	port map(
		clk => clk,
		reset => reset,
		upper_data_in => from_first_sorter_3,
		middle_data_in => from_first_pipeline_3,
		lower_data_in => from_second_pipeline_3,
		upper_data_out => from_second_sorter_1,
		middle_data_out => from_second_sorter_2,
		lower_data_out => from_second_sorter_3
	); 
	
	third_sorter_component : sorter_3
	generic map(
		channel_width => channel_width
	)
	port map(
		clk => clk,
		reset => reset,
		upper_data_in => from_first_sorter_2,
		middle_data_in => from_first_pipeline_2,
		lower_data_in => from_second_pipeline_2,
		upper_data_out => from_third_sorter_1,
		middle_data_out => from_third_sorter_2,
		lower_data_out => from_third_sorter_3
	);
	
	forth_sorter_component : sorter_3
	generic map(
		channel_width => channel_width
	)
	port map(
		clk => clk,
		reset => reset,
		upper_data_in => from_first_sorter_1,
		middle_data_in => from_first_pipeline_1,
		lower_data_in => from_second_pipeline_1,
		upper_data_out => from_forth_sorter_1,
		middle_data_out => from_forth_sorter_2,
		lower_data_out => from_forth_sorter_3
	);
	
	-- Forth line in diagram
	
	fifth_sorter_component : sorter_3
	generic map(
		channel_width => channel_width
	)
	port map(
		clk => clk,
		reset => reset,
		upper_data_in => from_second_sorter_3,
		middle_data_in => from_third_sorter_2,
		lower_data_in => from_forth_sorter_1,
		upper_data_out => dummy_out_1,
		middle_data_out => filtered_data_out,
		lower_data_out => dummy_out_2
	);
	
	original_data_out <= from_first_pipeline_2;
	
end architecture behaviour;
