--=============================================================================
-- Project: Video quality analysis using an FPGA
-- Author: Eirik T�rud Nordeng
--=============================================================================
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use ieee.numeric_std.all;

entity noise_metric is
	generic(
		channel_width : integer := 8;
		rows          : integer := 1080;
		cols          : integer := 1920
		);
	port (
		clk          : in  std_logic;
		reset       : in  std_logic;
		
		vsync : in std_logic;
		hsync : in std_logic;
		
		data_in   : in  integer range 0 to (2**channel_width - 1);
		
		metric  : out integer
		);
end noise_metric;

architecture behaviour of noise_metric is 
	
	constant channel_max_value : integer := 2**channel_width - 1;
	constant averaging_number_of_samples : integer := 32;
	
	-- Component declaration of the "median_filter(behaviour)" unit defined in
	-- file: "./../src/median_filter_3x3.vhd"
	component median_filter
		generic(
			channel_width : INTEGER := 8;
			rows : INTEGER := 1080;
			cols : INTEGER := 1920);
		port(
			clk : in STD_LOGIC;
			reset : in STD_LOGIC;
			vsync : in STD_LOGIC;
			hsync : in STD_LOGIC;
			data_in : in INTEGER range 0 to 2**channel_width-1;
			original_data_out : out INTEGER;
			filtered_data_out : out INTEGER);
	end component;
	for all: median_filter use entity work.median_filter(behaviour);
	
	-- Component declaration of the "averaging(behaviour)" unit defined in
	-- file: "./../src/averaging.vhd"
	component averaging
		generic(
			rows : INTEGER := 1080;
			cols : INTEGER := 1920;
			metric_max_value : INTEGER := 65535;
			number_of_samples : INTEGER := 8);
		port(
			clk : in STD_LOGIC;
			reset : in STD_LOGIC;
			enable : in STD_LOGIC;
			data_in : in INTEGER range 0 to metric_max_value;
			data_out : out INTEGER range 0 to metric_max_value);
	end component;
	for all: averaging use entity work.averaging(behaviour);
	
	
	signal original_data_out : integer range 0 to channel_max_value;	
	signal filtered_data_out : integer range 0 to channel_max_value;	
	signal raw_metric : integer range 0 to channel_max_value;
	signal from_first_pipeline_original : integer range 0 to channel_max_value;	
	signal from_first_pipeline_filtered : integer range 0 to channel_max_value;	
	signal from_second_pipeline : integer range 0 to channel_max_value;	
	signal enable_averaging : std_logic;
	
begin
	
	-- DEBUG!
	enable_averaging <= '1';
	
	filter_component : median_filter
	generic map(
		channel_width => channel_width,
		rows => rows,
		cols => cols
		)
	port map(
		clk => clk,
		reset => reset,
		vsync => vsync,
		hsync => hsync,
		data_in => data_in,
		original_data_out => original_data_out,
		filtered_data_out => filtered_data_out
		);
	
	first_pipelining_proc : process(clk)
	begin							
		if (rising_edge(clk)) then
			if (reset = '1') then
				from_first_pipeline_original <= 0;
				from_first_pipeline_filtered <= 0;
			else 
				from_first_pipeline_original <= original_data_out;				
				from_first_pipeline_filtered <= filtered_data_out;
			end if;
		end if;	
	end process first_pipelining_proc;
	
	raw_metric <= abs(from_first_pipeline_original - from_first_pipeline_filtered);
	
	second_pipelining_proc : process(clk)
	begin							
		if (rising_edge(clk)) then
			if (reset = '1') then
				from_second_pipeline <= 0;
			else  
				from_second_pipeline <= raw_metric;
			end if;
		end if;	
	end process second_pipelining_proc;
	
	average_component : averaging
	generic map(
		rows => rows,
		cols => cols,
		metric_max_value => channel_max_value,
		number_of_samples => averaging_number_of_samples
		)
	port map(
		clk => clk,
		reset => reset,
		enable => enable_averaging,
		data_in => from_second_pipeline,
		data_out => metric
		);

end architecture behaviour;
